@@ -1,8 +1,8 @@
-/*  $Id: CosmoSmashEngine.cpp,v 1.20 2002/12/27 02:52:47 sarrazip Exp $
+/*  $Id: CosmoSmashEngine.cpp,v 1.30 2007/06/20 03:02:43 sarrazip Exp $
     CosmoSmashEngine.cpp - A space rock shooting video game engine.
 
     cosmosmash - A space rock shooting video game.
-    Copyright (C) 2000, 2001, 2002 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2000-2004 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -22,7 +22,7 @@
 
 #include "CosmoSmashEngine.h"
 
-#include <gengameng/PixmapLoadError.h>
+#include <flatzebra/PixmapLoadError.h>
 
 #include <assert.h>
 #include <stdlib.h>
@@ -73,6 +73,7 @@
 #include "question0.xpm"
 #include "question1.xpm"
 
+using namespace std;
 
 #define __(s) (s)  /* Gettext-like macro */
 
@@ -249,7 +250,8 @@ removeNullElementsFromSpriteList(SpriteList &slist)
 
 CosmoSmashEngine::CosmoSmashEngine(
 		    long initialScore,
-		    bool _mirrorHyperspace, bool useSound, bool fullScreen)
+		    bool _mirrorHyperspace, bool useSound,
+		    bool fullScreen, bool automaticPlay)
 							throw(int, string)
   : GameEngine(theDrawingPixmapSize, "Cosmosmash", fullScreen),
   		// may throw string exception
@@ -295,18 +297,10 @@ CosmoSmashEngine::CosmoSmashEngine(
     groundHeight(0),
     mountainTopPos(0),
 
-    leftArrowPressed(false),
-    rightArrowPressed(false),
-    escapePressed(false),
-    spacePressed(false),
-    spacePressedBefore(false),
-    downArrowPressed(false),
-    downArrowPressedBefore(false),
-    letterEPressed(false),
-    letterEPressedBefore(false),
-    letterPPressed(false),
-    letterPPressedBefore(false),
-    ctrlPressed(false),
+    controller(automaticPlay
+		? static_cast<Controller *>(new AutoController())
+		: static_cast<Controller *>(new UserController())),
+    quitKS(SDLK_ESCAPE),
 
     initScore(initialScore),
     theScore(0),
@@ -324,6 +318,8 @@ CosmoSmashEngine::CosmoSmashEngine(
 
     theSoundMixer(NULL)
 {
+    this->useSound = useSound;
+
     assert(initScore >= 0);
     assert(theSDLScreen != NULL);
     assert(theSDLScreen->format != NULL);
@@ -448,19 +444,6 @@ CosmoSmashEngine::initializeMisc(bool useSound) throw(string)
     Throws an error message in a 'string' if an error occurs.
 */
 {
-    leftArrowPressed = false;
-    rightArrowPressed = false;
-    escapePressed = false;
-    spacePressed = false;
-    spacePressedBefore = false;
-    downArrowPressed = false;
-    downArrowPressedBefore = false;
-    letterEPressed = false;
-    letterEPressedBefore = false;
-    letterPPressed = false;
-    letterPPressedBefore = false;
-    ctrlPressed = false;
-
     theScore = initScore;
     thePeakScore = initScore;
     theLevel = getLevelNum(theScore);
@@ -511,14 +494,22 @@ CosmoSmashEngine::initializeMisc(bool useSound) throw(string)
 
 
     /*  Sound effects:
-	(ignored for now until good sound effects are found @sarrazip 20020330)
     */
     if (useSound)
     {
 	try
 	{
-	    string pkgsounddir = getDir(PKGSOUNDDIR, "PKGSOUNDDIR");
+	    theSoundMixer = NULL;
 	    theSoundMixer = new SoundMixer();  // may throw string
+	}
+	catch (const SoundMixer::Error &e)
+	{
+	    return;
+	}
+
+	try
+	{
+	    string pkgsounddir = getDir(PKGSOUNDDIR, "PKGSOUNDDIR");
 	    for (size_t i = 0; i < NUM_ROCK_HIT_SOUNDS; i++)
 	    {
 		string filename =
@@ -566,35 +557,27 @@ CosmoSmashEngine::setTimeBeforeNextQuestion()
 ///////////////////////////////////////////////////////////////////////////////
 
 
-/*virtual*/ void
+/*virtual*/
+void
 CosmoSmashEngine::processKey(SDLKey keysym, bool pressed)
 {
-    switch (keysym)
-    {
-        case SDLK_ESCAPE:  escapePressed = pressed; break;
-        case SDLK_LEFT:    leftArrowPressed = pressed; break;
-        case SDLK_RIGHT:   rightArrowPressed = pressed; break;
-        case SDLK_DOWN:    downArrowPressed = pressed; break;
-        case SDLK_SPACE:   spacePressed = pressed; break;
-        case SDLK_e:       letterEPressed = pressed; break;
-        case SDLK_p:       letterPPressed = pressed; break;
-	case SDLK_LCTRL:
-	case SDLK_RCTRL:   ctrlPressed = pressed; break;
+    if (UserController *uc = dynamic_cast<UserController *>(controller.get()))
+	uc->processKey(keysym, pressed);
 
-        default:           ;  // ignore other keys
-    }
+    quitKS.check(keysym, pressed);
 }
 
 
-/*virtual*/ bool
+/*virtual*/
+bool
 CosmoSmashEngine::tick()
 {
-    if (escapePressed)
+    if (quitKS.isPressed())
 	return false;
 
     if (paused)
     {
-	if (letterPPressed && !letterPPressedBefore)
+	if (controller->isPauseRequested())
 	{
 	    paused = false;
 	    displayPauseMessage(false);
@@ -618,13 +601,10 @@ CosmoSmashEngine::tick()
 	*/
 	if (numLives == 0)
 	{
-	    if (!spacePressed && spacePressedBefore
-			|| !letterEPressed && letterEPressedBefore)
+	    if (controller->isStartRequested(useGameExtensions))
 	    {
 		displayStartMessage(false);
 
-		useGameExtensions = letterEPressedBefore;
-
 		killAllAutomaticCharacters();
 		centerPlayerSprite();
 		theScore = initScore;
@@ -649,10 +629,7 @@ CosmoSmashEngine::tick()
 	    displayPauseMessage(true);
     }
 
-    spacePressedBefore = spacePressed;
-    downArrowPressedBefore = downArrowPressed;
-    letterEPressedBefore = letterEPressed;
-    letterPPressedBefore = letterPPressed;
+    controller->endOfTick();
 
     return true;
 }
@@ -704,13 +681,31 @@ CosmoSmashEngine::animatePlayer()
     Returns true if the game must continue, or false to have it stop.
 */
 {
-    if (letterPPressed && !letterPPressedBefore)
+    if (AutoController *ac = dynamic_cast<AutoController *>(controller.get()))
+    {
+	ac->beginGameState();
+
+	ac->setPlayerPos(playerSprite->getCenterPos());
+
+	SpriteList::iterator its;
+	for (its = rockSprites.begin(); its != rockSprites.end(); its++)
+	    ac->addRockPos((*its)->getCenterPos());
+	for (its = spinnerSprites.begin(); its != spinnerSprites.end(); its++)
+	    ac->addSpinnerPos((*its)->getCenterPos());
+	for (its = pulsarSprites.begin(); its != pulsarSprites.end(); its++)
+	    ac->addPulsarPos((*its)->getCenterPos());
+	for (its = saucerBulletSprites.begin(); its != saucerBulletSprites.end(); its++)
+	    ac->addPulsarPos((*its)->getCenterPos());
+
+	ac->endGameState();
+    }
+
+    if (controller->isResumeRequested())
     {
 	paused = true;
 	return true;
     }
 
-
     if (playerSprite->decTimeToLive() != 0)  // if player base exploding
     {
 	addToScore(0);  // patch: redraw score, because explosion munges it
@@ -729,15 +724,12 @@ CosmoSmashEngine::animatePlayer()
     */
     int ticksBetweenBullets =
 	   (playerBoostTicks > 0 && playerBoostType == CLOSER_BULLETS ? 3 : 6);
-    if (tickCount % ticksBetweenBullets == 0)
+    if (tickCount % ticksBetweenBullets == 0 && controller->isShootingActive())
     {
-	if (!ctrlPressed)
-	{
-	    Couple bbSize = baseBulletPA.getImageSize();
-	    Couple pos = playerPos + Couple((playerSize.x - bbSize.x) / 2, 0);
+	Couple bbSize = baseBulletPA.getImageSize();
+	Couple pos = playerPos + Couple((playerSize.x - bbSize.x) / 2, 0);
 
-	    fireBullet(pos);
-	}
+	fireBullet(pos);
     }
     if (playerBoostTicks > 0)
 	playerBoostTicks--;
@@ -750,7 +742,7 @@ CosmoSmashEngine::animatePlayer()
 
     playerSpeed.zero();
 
-    if (downArrowPressed && !downArrowPressedBefore)  // if hyperspace
+    if (controller->isHyperspaceRequested())
     {
 	Couple size = playerSprite->getSize();
 	int newX;
@@ -774,9 +766,9 @@ CosmoSmashEngine::animatePlayer()
 	return true;
     }
 
-    if (leftArrowPressed)
+    if (controller->isLeftMoveRequested())
 	playerSpeed.x = -PLAYER_SPEED;
-    if (rightArrowPressed)
+    if (controller->isRightMoveRequested())
 	playerSpeed.x = +PLAYER_SPEED;
 
     if (playerSpeed.isZero())
@@ -2022,7 +2014,8 @@ CosmoSmashEngine::displayPauseMessage(bool display)
     corresponding region if 'display' is false.
 */
 {
-    displayMessage(display ? "PAUSED -- press P to resume" : "");
+    if (display)
+	displayMessage(0, "PAUSED -- press P to resume");
 }
 
 
@@ -2033,54 +2026,32 @@ CosmoSmashEngine::displayStartMessage(bool display)
     corresponding region if 'display' is false.
 */
 {
-    if (!display)
-	displayMessage("");
-    else
+    if (display)
     {
 	static const Couple fontdim = getFontDimensions();
 
-	displayMessage("Cosmosmash " VERSION " - by Pierre Sarrazin",
-			scoreAreaPos.y - fontdim.y);
-	displayMessage("       Distributed under the GNU GPL       ",
-			scoreAreaPos.y);
+	displayMessage(0, "Cosmosmash " VERSION " - by Pierre Sarrazin   ");
+	displayMessage(1, "Press SPACE to start a standard game    ");
+	displayMessage(2, "Press E to start a game with extensions ");
 
+	displayMessage(4, "During game: move with left/right arrows");
+	displayMessage(5, "and press down arrow for hyperspace;    ");
+	displayMessage(6, "press P to pause the game.              ");
 
-	static const char *lines[] =
-	{
-	    "Press SPACE to start a standard game    ",
-	    "Press E to start a game with extensions ",
-	    "                                        ",
-	    "During game: move with left/right arrows",
-	    "and press down arrow for hyperspace;    ",
-	    "press P to pause the game.              ",
-	    NULL
-	};
-	static int numLines = 6;  // CAUTION: must be right number of lines
-
-	static int y = (groundPos - fontdim.y * numLines) / 2;
-	for (size_t i = 0; lines[i] != NULL; i++)
-	    displayMessage(lines[i], y + i * fontdim.y);
+	if (useSound && theSoundMixer == NULL)
+	    displayMessage(8, "*** Sound device not available ***      ");
     }
 }
 
 
 void
-CosmoSmashEngine::displayMessage(const char *msg)
-{
-    displayMessage(msg, scoreAreaPos.y);
-}
-
-
-void
-CosmoSmashEngine::displayMessage(const char *msg, int y)
-/*  Displays the string 'msg' at a horizontally centered position
-    and at vertical position 'y'.
-    The font used is that of gengameng.
-*/
+CosmoSmashEngine::displayMessage(int row, const char *msg)
 {
     static const Couple fontdim = getFontDimensions();
 
-    int x = (theScreenSizeInPixels.x - strlen(msg) * fontdim.x) / 2;
+    const size_t len = strlen(msg);
+    int x = (theScreenSizeInPixels.x - len * fontdim.x) / 2;
+    int y = theScreenSizeInPixels.y + (row - 26) * fontdim.y;
 
     writeString(msg, Couple(x, y));
 }
@@ -2095,15 +2066,17 @@ CosmoSmashEngine::playShortExplosionSound()
 
 
 void
-CosmoSmashEngine::playSoundEffect(SoundMixer::Chunk &wb)
+CosmoSmashEngine::playSoundEffect(SoundMixer::Chunk &chunk)
 {
-    /*  'theSoundMixer' is still NULL if the sound effects were not wanted.
+    /*  'theSoundMixer' is still NULL if the sound effects were not wanted
+	or if the sound device is not available.
+	Also, there are no sounds during the demo mode.
     */
-    if (theSoundMixer != NULL)
+    if (theSoundMixer != NULL && numLives != 0)
     {
 	try
 	{
-	    theSoundMixer->playChunk(wb);
+	    theSoundMixer->playChunk(chunk);
 	}
 	catch (const SoundMixer::Error &e)
 	{
