@@ -1,8 +1,8 @@
-/*  $Id: CosmoSmashEngine.h,v 1.13 2002/08/01 02:10:38 sarrazip Exp $
+/*  $Id: CosmoSmashEngine.h,v 1.23 2007/06/20 03:02:43 sarrazip Exp $
     CosmoSmashEngine.h - A space rock shooting video game engine.
 
     cosmosmash - A space rock shooting video game.
-    Copyright (C) 2000, 2001, 2002 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2000-2004 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -23,14 +23,19 @@
 #ifndef _H_CosmoSmashEngine
 #define _H_CosmoSmashEngine
 
-#include "SoundMixer.h"
+#include "UserController.h"
+#include "AutoController.h"
 
-#include <gengameng/GameEngine.h>
-#include <gengameng/Sprite.h>
+#include <flatzebra/GameEngine.h>
+#include <flatzebra/Sprite.h>
+#include <flatzebra/SoundMixer.h>
+#include <flatzebra/KeyState.h>
 
 #include <string>
+#include <vector>
+#include <memory>
 
-using namespace std;
+using namespace flatzebra;
 
 
 class CosmoSmashEngine : public GameEngine
@@ -39,8 +44,9 @@ class CosmoSmashEngine : public GameEngine
 {
 public:
     CosmoSmashEngine(long initialScore,
-    		bool mirrorHyperspace, bool useSound, bool fullScreen)
-    							throw(int, string);
+			bool mirrorHyperspace, bool useSound,
+			bool fullScreen, bool automaticPlay)
+    							throw(int, std::string);
     /*  See base class.
 
 	Throws a string or an integer code if an error occurs.
@@ -73,7 +79,7 @@ protected:
 
 private:
 
-    typedef vector<Couple> CoupleList;
+    typedef std::vector<Couple> CoupleList;
 
     enum PlayerBoostType
     {
@@ -83,6 +89,7 @@ private:
 
     bool paused;
     unsigned long tickCount;
+    bool useSound;
 
     // SDL color values (see SDL_MapRGB()):
     Uint32 blackColor;
@@ -127,13 +134,8 @@ private:
     int groundHeight;  // height of ground in pixels
     int mountainTopPos;  // y position of highest possible pixel for mountains
 
-    bool leftArrowPressed, rightArrowPressed;
-    bool escapePressed;
-    bool spacePressed, spacePressedBefore;
-    bool downArrowPressed, downArrowPressedBefore;
-    bool letterEPressed, letterEPressedBefore;
-    bool letterPPressed, letterPPressedBefore;
-    bool ctrlPressed;
+    std::auto_ptr<Controller> controller;
+    KeyState quitKS;
 
     long   initScore;     // score at which a game starts (determines level)
 
@@ -171,7 +173,7 @@ private:
     */
     void fireBullet(Couple pos);
     void initializeSprites() throw(PixmapLoadError);
-    void initializeMisc(bool useSound) throw(string);
+    void initializeMisc(bool useSound) throw(std::string);
     void setTimeBeforeNextPulsar();
     void setTimeBeforeNextSaucer();
     void setTimeBeforeNextQuestion();
@@ -179,7 +181,7 @@ private:
     void readKeyboardCommands();
     Sprite *createExplosionSprite(Couple posOfCenterOfExplosion,
 				    int timeToLive);
-    pair<Sprite *, Sprite *> splitBigRock(const Sprite &bigRock) const;
+    std::pair<Sprite *, Sprite *> splitBigRock(const Sprite &bigRock) const;
     void makeSpriteExplode(const Sprite *target,
 			    const Sprite *bullet,
 			    long points);
@@ -191,12 +193,11 @@ private:
     void killAllAutomaticCharacters();
     void killPlayerBase();
     bool playerIsActive() const;
-    pair<bool, size_t> animateFallingObjects(SpriteList &sprites);
+    std::pair<bool, size_t> animateFallingObjects(SpriteList &sprites);
     void animateSaucers();
     void displayPauseMessage(bool display);
     void displayStartMessage(bool display);
-    void displayMessage(const char *msg);
-    void displayMessage(const char *msg, int y);
+    void displayMessage(int row, const char *msg);
     bool detectCollisionsWithBullet(const Sprite &bullet,
 				    SpriteList &slist,
 				    int scoreOnCollision);
