@@ -1,8 +1,8 @@
-/*  $Id: sdlmain.cpp,v 1.9 2002/06/22 16:42:15 sarrazip Exp $
+/*  $Id: sdlmain.cpp,v 1.14 2007/06/20 03:02:43 sarrazip Exp $
     sdlmain.cpp - main() function for Cosmosmash
 
     cosmosmash - A space rock shooting video game.
-    Copyright (C) 2000, 2001, 2002 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2000-2004 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -30,11 +30,13 @@
 #include <errno.h>
 #include <time.h>
 
-
-const int defaultMillesecondsPerFrame = 58;
+using namespace std;
 
 
-#ifdef __GNUG__
+static const int defaultMillesecondsPerFrame = 58;
+
+
+#ifdef HAVE_GETOPT_LONG
 #include <getopt.h>
 
 
@@ -47,6 +49,7 @@ static struct option knownOptions[] =
     { "mirror-hyperspace", no_argument,       NULL, 'y' },
     { "no-sound",          no_argument,       NULL, 'n' },
     { "full-screen",       no_argument,       NULL, 'f' },
+    { "auto",              no_argument,       NULL, 'a' },
 
     { NULL, 0, NULL, 0 }  // marks the end
 };
@@ -83,6 +86,7 @@ displayHelp()
 "                     instead of begin random [defaut=random]\n"
 "--no-sound           Disable sound effects [default is to enable them]\n"
 "--full-screen        Attempt to use full screen mode [default is window mode]\n"
+"--auto               Automatic mode (the computer plays the game itself)\n"
 "\n"
     ,
     defaultMillesecondsPerFrame);
@@ -99,9 +103,10 @@ int main(int argc, char *argv[])
     bool mirrorHyperspace = false;
     bool useSound = true;
     bool fullScreen = false;
+    bool automaticPlay = false;
 
 
-    #ifdef __GNUG__
+    #ifdef HAVE_GETOPT_LONG
 
     /*  Interpret the command-line options:
     */
@@ -157,11 +162,18 @@ int main(int argc, char *argv[])
 		fullScreen = true;
 		break;
 
+	    case 'a':
+		automaticPlay = true;
+		break;
+
 	    case 'v':
 		displayVersionNo();
-		return EXIT_FAILURE;
+		return EXIT_SUCCESS;
 
 	    case 'h':
+		displayHelp();
+		return EXIT_SUCCESS;
+
 	    default:
 		displayHelp();
 		return EXIT_FAILURE;
@@ -186,7 +198,8 @@ int main(int argc, char *argv[])
     try
     {
 	CosmoSmashEngine theCosmoSmashEngine(
-		    initScore, mirrorHyperspace, useSound, fullScreen);
+				initScore, mirrorHyperspace, useSound,
+				fullScreen, automaticPlay);
 	theCosmoSmashEngine.run(millisecondsPerFrame);
     }
     catch (const string &s)
